/*******************************************************************************
 benten.cat.glossary.engine.omegat - Benten default glossary engine.
 Copyright (C) 2009 Information-technology Promotion Agency, Japan.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, see <http://www.gnu.org/licenses>.

 Linking benten.cat.glossary.engine.omegat statically or dynamically
 with other modules is making a combined work based on 
 benten.cat.glossary.engine.omegat. Thus, the terms and conditions
 of the GNU General Public License cover the whole combination.

 In addition, as a special exception, the copyright holders of
 benten.cat.glossary.engine.omegat give you permission to combine
 benten.cat.glossary.engine.omegat with free software programs or
 libraries that are released under the GNU LGPL and with code included
 in the standard release of benten.cat.glossary.core under the EPL
 (or modified versions of such code, with unchanged license). You may
 copy and distribute such a system following the terms of the GNU GPL
 for benten.cat.glossary.engine.omegat and the licenses of the other
 code concerned, provided that you include the source code of that
 other code when and as the GNU GPL requires distribution of source code.

 Note that people who make modified versions of
 benten.cat.glossary.engine.omegat are not obligated to grant this special
 exception for their modified versions; it is their choice whether to do so.
 The GNU General Public License gives permission to release a modified
 version without this exception; this exception also makes it possible to
 release a modified version which carries forward this exception.
 *******************************************************************************/
package benten.cat.glossary.engine.omegat.ui;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.jface.preference.IPreferenceStore;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.BentenGlossaryDriver;
import benten.cat.glossary.core.BentenGlossaryDriverManager;
import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.data.AbstractBentenGlossaryDataTsvPlugin;
import benten.cat.glossary.engine.AbstractBentenGlossaryEnginePlugin;
import benten.cat.glossary.engine.omegat.DefaultGlossaryDriver;
import benten.cat.tm.engine.omegat.ui.messages.OmegatGlossaryPluginMessages;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;

/**
 * Benten システムのデフォルトの用語集エンジン・プラグインである OmegaT 用語集プラグイン。
 * 
 * <UL>
 * <LI>このプラグインを Eclipse 上で有効化するためのプラグイン・クラスです。
 * </UL>
 * 
 * ★基本設計「共通機能: 用語対訳集エンジン機能」に対応します。
 * 
 * @author YAMAMOTO Koji
 */
public class OmegatGlossaryPlugin extends AbstractBentenGlossaryEnginePlugin {
	/**
	 * Benten デフォルト用語集エンジン・プラグインのためのメッセージ。
	 */
	protected static final OmegatGlossaryPluginMessages fMsg = new OmegatGlossaryPluginMessages();

	/**
	 * デフォルト用語集の名称。
	 */
	private static final String DEFAULT_GLOSSARY = "Default Glossary"; //$NON-NLS-1$

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	@Override
	public BentenGlossaryDriver getDriver() {
		try {
			// 自らのドライバーを自己登録します。
			Class.forName("benten.cat.glossary.engine.omegat.DefaultGlossaryDriver"); //$NON-NLS-1$
		} catch (final ClassNotFoundException e) {
			log(fMsg.getGetDriverClassNotFoundException());
			log(e);
		}

		return BentenGlossaryDriverManager.getDriver(DefaultGlossaryDriver.DRIVER_NAME);
	}

	@Override
	public BentenGlossaryEngine getEngine() {
		if (fEngine == null) {
			fEngine = getDriver().getEngineInstance();
		}
		return fEngine;
	}

	@Override
	public void load() {
		getDefault().log(fMsg.getLoadI001());

		getEngine().unload();

		final File tsvDir = getTsvDir();
		try {
			getEngine().loadTsv(tsvDir);
		} catch (final IOException e) {
			getDefault().log(e);
		}
	}

	/**
	 * 拡張ポイントなどから TSV ディレクトリーを取得します。
	 * 
	 * @return TSV ディレクトリー。必ず null 以外が戻ります。
	 */
	@SuppressWarnings("static-access")
	private File getTsvDir() {
		File result = null;

		final IPreferenceStore store = BentenPreference.getStore();
		final boolean useTsvLocation = store.getBoolean(Preference.USE_TSV_LOCATION.name());
		if (useTsvLocation) {
			final String tsvFolder = store.getString(Preference.TSV_LOCATION.name());
			result = new File(tsvFolder);
		} else {
			result = getTsvDataPlugin().getDefault().getFile("/glossary"); //$NON-NLS-1$
		}
		getDefault().log(fMsg.getGetTsvDirI001(result.toString()));

		if (result.exists() == false) {
			throw new IllegalArgumentException(fMsg.getGetTsvDirE001(result.getAbsolutePath()));
		}
		if (result.isDirectory() == false) {
			throw new IllegalArgumentException(fMsg.getGetTsvDirE002(result.getAbsolutePath()));
		}

		return result;
	}

	/**
	 * 拡張ポイントから用語集データ TSV プラグインを取得します。
	 * 
	 * @return 用語集データ TSV プラグイン。必ず null 以外が戻ります。
	 */
	private AbstractBentenGlossaryDataTsvPlugin getTsvDataPlugin() {
		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.glossary.data.tsv"); //$NON-NLS-1$

		if (point == null) {
			throw new IllegalArgumentException(fMsg.getGetTsvDataPluginE001());
		}

		final Map<String, AbstractBentenGlossaryDataTsvPlugin> tsvMap = new HashMap<String, AbstractBentenGlossaryDataTsvPlugin>();

		for (IExtension extension : point.getExtensions()) {
			for (IConfigurationElement cfgElem : extension.getConfigurationElements()) {
				try {
					final String name = cfgElem.getAttribute("name"); //$NON-NLS-1$
					final Object tsvPluginClass = cfgElem.createExecutableExtension("class"); //$NON-NLS-1$
					if (tsvPluginClass instanceof Plugin) {
						tsvMap.put(name, (AbstractBentenGlossaryDataTsvPlugin) tsvPluginClass);
					}
				} catch (final CoreException e) {
					getDefault().log(e);
				}
			}
		}

		final AbstractBentenGlossaryDataTsvPlugin tsvPlugin = tsvMap.get(DEFAULT_GLOSSARY);
		if (tsvPlugin == null) {
			throw new IllegalArgumentException(fMsg.getGetTsvDataPluginE002());
		}

		return tsvPlugin;
	}
}
